@@ -1,6 +1,6 @@
 /*
  * music player command (mpc)
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2015 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@
 int
 stdinToArgArray(char ***array)
 {
-	List *list = makeList(NULL);
+	List *list = makeList();
 	char buffer[4096];
 
 	while (fgets(buffer, sizeof(buffer), stdin)) {
@@ -172,7 +172,7 @@ parse_songnum(const char *str, int *ret)
 	char *endptr;
 	int song = strtol(str, &endptr, 10);
 
-	if (str == endptr || (*endptr != ')' && *endptr != '\0') || song < 0)
+	if (str == endptr || (*endptr != ')' && *endptr != '\0') || song < 1)
 		return false;
 
 	*ret = song;
