@@ -1,6 +1,6 @@
 /*
  * music player command (mpc)
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2015 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,7 +65,7 @@ static struct command {
 	{ "current", 0, 0, 0, cmd_current,
 	  "", "Show the currently playing song"},
 	{"del",         0,   -1,  1,    cmd_del,         "<position>", "Remove a song from the current playlist"},
-	{"play",        0,   -1,  2,    cmd_play,        "[<position>]", "Start playing at <position> (default: 1)"},
+	{"play",        0,   1,   2,    cmd_play,        "[<position>]", "Start playing at <position>"},
 	{"next",        0,   0,   0,    cmd_next,        "", "Play the next song in the current playlist"},
 	{"prev",        0,   0,   0,    cmd_prev,        "", "Play the previous song in the current playlist"},
 	{"pause",       0,   0,   0,    cmd_pause,       "", "Pauses the currently playing song"},
@@ -98,9 +98,10 @@ static struct command {
 	{"single",      0,   1,   0,    cmd_single,      "<on|off>", "Toggle single mode, or specify state"},
 	{"consume",     0,   1,   0,    cmd_consume,     "<on|off>", "Toggle consume mode, or specify state"},
 	{"search",      2,   -1,  0,    cmd_search,      "<type> <query>", "Search for a song"},
+	{"searchadd",   2,   -1,  0,    cmd_searchadd,   "<type> <query>", "Search songs and add them to the current playlist"},
 	{"find",        2,   -1,  0,    cmd_find,        "<type> <query>", "Find a song (exact match)"},
-	{"findadd", 2, -1, 0, cmd_findadd, "<type> <query>",
-	 "Find songs and add them to the current playlist"},
+	{"findadd",     2,   -1,  0,    cmd_findadd,     "<type> <query>", "Find songs and add them to the current playlist"},
+	{"searchplay",  1,   1,   0,    cmd_searchplay,  "<pattern>", "Find and play a song in the current playlist"},
 	{"list",        1,   -1,  0,    cmd_list,        "<type> [<type> <query>]", "Show all tags of <type>"},
 	{"crossfade",   0,   1,   0,    cmd_crossfade,   "[<seconds>]", "Set and display crossfade settings"},
 #if LIBMPDCLIENT_CHECK_VERSION(2,4,0)
@@ -214,11 +215,24 @@ setup_connection(void)
 static struct command *
 find_command(const char *name)
 {
-	for (unsigned i = 0; mpc_table[i].command != NULL; ++i)
-		if (strcmp(name, mpc_table[i].command) == 0)
-			return &mpc_table[i];
+	unsigned n_matches = 0;
+	size_t name_length = strlen(name);
+	struct command *command = NULL;
 
-	return NULL;
+	for (unsigned i = 0; mpc_table[i].command != NULL; ++i) {
+		if (memcmp(name, mpc_table[i].command, name_length) == 0) {
+			command = &mpc_table[i];
+			++n_matches;
+
+			if (command->command[name_length] == 0)
+				/* exact match */
+				return &mpc_table[i];
+		}
+	}
+
+	return n_matches == 1
+		? command
+		: /* ambiguous or nonexistent */ NULL;
 }
 
 /* check arguments to see if they are valid */
